#!/usr/bin/python3

# imports for config file
import argparse
import yaml
import os

# own class and file imports
from classes.Document_Collection import *
from classes.BPMN import *
from classes.Process import *
from classes.Pair import *
from classes.SimilarityComputer import *
from functions import *

# imports for bpmnparser
from xml.dom import minidom
from collections import defaultdict


GATEWAYS = ["parallelGateway", "exclusiveGateway", "complexGateway", "eventBasedGateway", "inclusiveGateway"]
EVENTS = ["intermediateCatchEvent", "intermediateThrowEvent", "boundaryEvent"]

nlp = spacy.load('en_core_web_lg')
#nlp = spacy.load('en')     #debugging only as this does not contain word vectors
         

## parse configurations from config file ########################### START
parser = argparse.ArgumentParser()
parser.add_argument('-c', '--config', help="Path to Config File.")
args = parser.parse_args()

with open(args.config, 'r') as stream:
  try:
    conf = yaml.full_load(stream)
  except yaml.YAMLError as exc:
    print(exc)

for k,v in conf.items():
  vars(args)[k]=v


try:
  with open(args.signalwords) as f:
    signalwords = f.read().splitlines()
except FileNotFoundError:
  print("Wrong file or file path to signalwords. Check config file.")
  quit()
  
  
try:
  with open(args.sequencemarkers) as f:
    sequencemarkers = f.read().splitlines()
except FileNotFoundError:
  print("Wrong file or file path to sequencemarkers. Check config file.")
  quit()

  
try:
  with open(args.stopwords) as f:
    stopwords = f.read().splitlines()
except FileNotFoundError:
  print("Wrong file or file path to stop words. Check config file.")
  quit()
  
################################################################# END


## parse Regulations ############################################ START
paragraphs = dict()
files = os.listdir(args.docdir)
try:
  for fi in files:
    with open(args.docdir+fi) as f:
      paragraphs[re.sub('\.txt', '', fi)] = f.read()
except FileNotFoundError:
  print("Wrong file or file path to docdir. Check config file.")
  quit()


document_collection = Document_Collection(int(args.collectionid), paragraphs, signalwords, sequencemarkers, stopwords, nlp, args.only_constraints)
################################################################# END


## parse BPMN files ############################################ START
try:
  bpmn_models = list()
  resource_set = set()
  for f in os.listdir(args.modeldir):
    if f.startswith('.'): # handle hidden files on OSX
      continue
    
    doc = minidom.parse(args.modeldir+f)
    processes_dom_elements = doc.getElementsByTagName("process")

    processes = list()
    for p in processes_dom_elements:
      _id = p.getAttribute("id")
      participant = p.getAttribute("name")
      start_events = p.getElementsByTagName("startEvent")
      end_events = p.getElementsByTagName("endEvent")
      tasks = p.getElementsByTagName("task")
      flows = p.getElementsByTagName("sequenceFlow")
      gateways = defaultdict(list)
      for g in GATEWAYS:
        gateways[g] = p.getElementsByTagName(g)
      events = defaultdict(list)
      for e in EVENTS:
        events[e] = p.getElementsByTagName(e)
      resource_set.add(participant.lower())
      processes.append(Process(_id, nlp, stopwords, participant, start_events, end_events, tasks, gateways, events, flows))    
    bpmn_models.append(BPMN(f, processes))
except FileNotFoundError:
  print("Wrong file or file path to docdir. Check config file.")
  quit()
################################################################# END


# initialize SimilarityComputer object 
sim = SimilarityComputer(nlp, bpmn_models, document_collection, stopwords, args.spacy_cache)

# build all possible model paragraph combinations and sort by model
pairs_per_model = defaultdict(list)
for model in bpmn_models:
  for paragraph in document_collection.paragraphs:
    pairs_per_model[model._id].append(Pair(nlp, sim, model, paragraph, resource_set, args.gamma, args.delta))

evaluation_csv(args.targetdir, pairs_per_model)
